/****************************************************************************
 * This file is part of Hawaii.
 *
 * Copyright (C) 2015-2016 Pier Luigi Fiorini
 *
 * Author(s):
 *    Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
 *
 * $BEGIN_LICENSE:LGPL$
 *
 * This file may be used under the terms of the GNU Lesser General Public
 * License version 2.1 or later as published by the Free Software Foundation
 * and appearing in the file LICENSE.LGPLv21 included in the packaging of
 * this file.  Please review the following information to ensure the
 * GNU Lesser General Public License version 2.1 requirements will be
 * met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * Alternatively, this file may be used under the terms of the GNU General
 * Public License version 2.0 or later as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPLv2 included in the
 * packaging of this file.  Please review the following information to ensure
 * the GNU General Public License version 2.0 requirements will be
 * met: http://www.gnu.org/licenses/gpl-2.0.html.
 *
 * $END_LICENSE$
 ***************************************************************************/

#ifndef GREENISLAND_HOMEAPPLICATION_P_H
#define GREENISLAND_HOMEAPPLICATION_P_H

#include <QtCore/QFileInfo>
#include <QtCore/private/qobject_p.h>

#include <GreenIsland/Server/HomeApplication>

namespace GreenIsland {

namespace Server {

static int convertPermission(const QFileInfo &fileInfo)
{
    int p = 0;

    if (fileInfo.permission(QFile::ReadUser))
        p += 400;
    if (fileInfo.permission(QFile::WriteUser))
        p += 200;
    if (fileInfo.permission(QFile::ExeUser))
        p += 100;
    if (fileInfo.permission(QFile::ReadGroup))
        p += 40;
    if (fileInfo.permission(QFile::WriteGroup))
        p += 20;
    if (fileInfo.permission(QFile::ExeGroup))
        p += 10;
    if (fileInfo.permission(QFile::ReadOther))
        p += 4;
    if (fileInfo.permission(QFile::WriteOther))
        p += 2;
    if (fileInfo.permission(QFile::ExeOther))
        p += 1;

    return p;
}

class GREENISLANDSERVER_EXPORT HomeApplicationPrivate : public QObjectPrivate
{
    Q_DECLARE_PUBLIC(HomeApplication)
public:
    HomeApplicationPrivate()
        : initialized(false)
        , running(false)
        , notify(true)
        , engine(Q_NULLPTR)
    {
    }

    void verifyXdgRuntimeDir();

    bool initialized;
    bool running;
    bool notify;
    QString screenConfiguration;
    QQmlApplicationEngine *engine;
};

} // namespace Server

} // namespace GreenIsland

#endif // GREENISLAND_HOMEAPPLICATION_P_H

